<?php

namespace Drupal\entity_template_select\Plugin\Field\FieldWidget;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityBase;
use Drupal\Core\Entity\Plugin\DataType\EntityAdapter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'template_select_widget' widget.
 *
 * @FieldWidget(
 *   id = "template_select_widget",
 *   label = @Translation("Template select widget"),
 *   field_types = {
 *     "template_select_type"
 *   }
 * )
 */
class TemplateSelectWidget extends WidgetBase {
  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $systemTheme;


  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'sidebar' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element['sidebar'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Place field in sidebar'),
      '#default_value' => $this->getSetting('sidebar'),
      '#description' => $this->t('If checked, the field will be placed in the sidebar on entity forms.'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    if ($this->getSetting('sidebar')) {
      $summary[] = $this->t('Use sidebar: Yes');
    }
    else {
      $summary[] = $this->t('Use sidebar: No');
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, AccountProxyInterface $current_user, ImmutableConfig $system_theme, FileSystemInterface $file_system) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->currentUser = $current_user;
    $this->systemTheme = $system_theme;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('current_user'),
      $container->get('config.factory')->get('system.theme'),
      $container->get('file_system')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $entityAdapter = $items->getParent();
    if ($entityAdapter instanceof EntityAdapter) {
      $entity = $entityAdapter->getValue();
      if ($entity instanceof EntityBase) {
        $entityType = $entity->getEntityTypeId();
        if ($this->currentUser->hasPermission('allow users to pick ' . $entityType . 'templates')) {
          $bundle = $entity->bundle();
          $options = $this->getTemplates($entityType, $bundle);
          $element['value'] = [
            '#type' => 'select',
            '#options' => $options,
            '#empty_option' => $this->t('- Select -'),
            '#title' => $this->t('Template Select'),
            '#description' => $this->t('Select template apply for this content'),
            '#default_value' => isset($items[$delta]->value) ?
            $items[$delta]->value : [],
          ];

          // If the "sidebar" option was checked on the field widget, put the
          // form element into the form's "advanced" group. Otherwise, let it
          // default to the main field area.
          $sidebar = $this->getSetting('sidebar');
          if ($sidebar) {
            $element += [
              '#type' => 'details',
              '#open' => FALSE,
              '#group' => 'advanced',
            ];
          }

        }
      }
    }

    return $element;
  }

  /**
   * Fetch an array of possible templates for an entity type, bundle, and theme.
   *
   * @param string $entity_type
   *   The entity type (e.g. 'node').
   * @param string $bundle
   *   The bundle of the entity type (e.g. 'article').
   *
   * @return array
   *   An array of available templates.
   */
  public function getTemplates($entity_type, $bundle) {
    $theme = $this->systemTheme->get('default');

    // Get the path to the theme directory we want.
    $theme_path = drupal_get_path('theme', $theme);
    if (empty($theme_path)) {
      return [];
    }

    // Detect templates in your theme like "node--content-type--tp*.twig.php".
    $options = [];

    $entity_type_hyph = str_replace('_', '-', $entity_type);
    $bundle_hyph = str_replace('_', '-', $bundle);
    $regex = '/' . $entity_type_hyph . '--' . $bundle_hyph . '--+.*tp.html.twig$/';

    $files = $this->fileSystem->scanDirectory(DRUPAL_ROOT . '/' . $theme_path, $regex);
    foreach ($files as $file) {
      $key = $this->fileSystem->basename($file->filename, '.html.twig');
      $options[$key] = $this->getTemplateName($file->uri, $key);
    }

    return $options;
  }

  /**
   * Get a human-readable name from a template if possible.
   *
   * @param string $uri
   *   The URI to the template file.
   * @param string $default
   *   The default name to return if a human-readable name was not found in the
   *   template file.
   *
   * @return string
   *   The human-readable name of the template, or $default if not found.
   */
  public function getTemplateName($uri, $default) {
    if (is_file($uri) && $contents = file_get_contents($uri)) {
      if (preg_match('/\{\#Template Name:(.*?)\#\}/s', $contents, $results)) {
        // A clean name was found. Save the result.
        return trim($results[1]);
      }
    }

    return $default;
  }

}
